
from escape.core.data import data_obj
from escape.core.objects import kernel_obj, functor_obj, kernel as kernel_fac
from escape.core.model import model_obj
from escape.core.optimizer import optimizer_obj
from escape.core.model import modelstack_obj, modelstack as esc_modelstack
from escape.core.regressor import regressor_obj

from escape.scattering.geometry import geometry_obj
from escape.scattering.potential import potential_obj
from escape.scattering.layer import multilayer_obj


from escape.utils.widgets.parameter import *
from escape.utils.widgets.data import *
from escape.utils.widgets.kernel import *
from escape.utils.widgets.model import *
from escape.utils.widgets.modelstack import *
from escape.utils.widgets.optimizer import *
from escape.utils.widgets.shape import *
from escape.utils.widgets.regressor import *
from escape.utils.widgets.profile import *

__all__ = ["show"]


def show(obj, *args, **kwargs):
    if isinstance(obj, parameter_obj):
        if obj.independent:
            return display(IndependentParameterWidget(obj, *args, **kwargs))
        else:
            return display(DependentParameterWidget(obj, *args, **kwargs))
    elif isinstance(obj, functor_obj):
        return display(KernelObjectLayout(kernel_fac(obj.name, obj, True), *args, **kwargs))
    elif isinstance(obj, kernel_obj):
        return display(KernelObjectLayout(obj, *args, **kwargs))
    elif isinstance(obj, data_obj):
        return display(DataObjectLayout(obj, *args, **kwargs))
    elif isinstance(obj, model_obj):
        return display(ModelObjectLayout(obj, *args, **kwargs))
    elif isinstance(obj, (tuple, list)) and all(isinstance(item, model_obj) for item in obj):
        obj = esc_modelstack("", obj)
        return show(obj, *args, **kwargs)
    elif isinstance(obj, modelstack_obj):
        mode = kwargs.get("mode", None)
        if mode is None:
            return display(MSObjectLayout(obj, *args, **kwargs))
        elif mode == "single":
            return display(MSSinglePlotObjectLayout(obj, *args, **kwargs))
    elif isinstance(obj, optimizer_obj):
        return display(OptimizerObjectLayout(obj, *args, **kwargs))
    elif isinstance(obj, regressor_obj):
        return display(RegressorObjectLayout(obj, *args, **kwargs))
    elif isinstance(obj, (geometry_obj, potential_obj)):
        return display(ShapeLayout(obj, *args, **kwargs))
    elif isinstance(obj, (multilayer_obj)):
        return display(ProfileLayout(obj, *args, **kwargs))
    
    raise TypeError("unsupported object type: {0}".format(type(obj)))