import numpy as np

from escape.utils.widgets.base import PlotObjectLayout
from escape.utils.widgets.plot import Plot2dWidget, MapWidget, PLOT_TYPE_KW, \
    PLOT2D_TYPE, MAP_TYPE, PLOT_TYPES


class KernelObjectLayout(PlotObjectLayout):
    
    plot2d_widget_class = Plot2dWidget
    
    def __init__(self, escobj, **kwargs):
        dims = escobj.domain_size
        coordinates = kwargs.get("coordinates", None)
        if coordinates is None:
            if dims == 1:
                coordinates = np.linspace(-10, 10, 300, dtype=float).copy()
            else:
                raise ValueError("Coordinates argument is required for n-dimensional kernel")
        coordinates = np.array(coordinates, dtype=float).flatten()
        self.coordinates = coordinates
        super(KernelObjectLayout, self).__init__(escobj, **kwargs)
    
    def update(self, *args):
        escobj = self.escobj
        dims = escobj.domain_size
        xarr = self.coordinates
        yarr = np.zeros(shape=(int(xarr.shape[0] / dims),), dtype=float)
        escobj(xarr, yarr)
        if dims == 1: 
            self.center.update(xarr, yarr)
        else:
            plot_type = self._plot_type
            coordidx = self._coordidx
            if plot_type == PLOT2D_TYPE:
                if not isinstance(coordidx, int):
                    raise TypeError("Unexpected type of coordinate index, expected int, got {0}".format(
                        type(coordidx)
                    ))
                xarr = xarr[coordidx::dims]
                self.center.update(xarr, yarr)
            elif plot_type == MAP_TYPE:
                if dims > 2:
                    if not isinstance(coordidx, (list, tuple)):
                        raise TypeError("Unexpected type of coordinate index, expected list or tuple, got {0}".format(
                            type(coordidx)
                        ))
                    
                    msk = np.zeros(shape=xarr.shape, dtype=bool)
                    
                    for idx in coordidx:
                        msk[idx::dims] = True
                    xarr = xarr[msk]
                rows = self._rows
                cols = int(len(yarr) / rows)
                if rows * cols != len(yarr):
                    raise ValueError("Cannot build map with given number of rows")
                yarr = yarr.reshape((rows, cols))
                xarr = xarr.reshape((int(len(xarr) / 2), 2))
                xmin, xmax = np.min(xarr[:, 0]), np.max(xarr[:, 0])
                ymin, ymax = np.min(xarr[:, 1]), np.max(xarr[:, 1])
                self.center.update(yarr, extent=(xmin, xmax, ymin, ymax))
            else:
                raise ValueError("Wrong type of plot, expected one of [{0}], got {1}".format(
                    ", ".join(PLOT_TYPES), plot_type
                ))

