import numpy as np
import escape as esc
from escape.utils.widgets.kernel import KernelObjectLayout

SLD0RE_TYPE = "sld0re"
SLD0IM_TYPE = "sld0im"
SLDM_TYPE   = "sldm"


class ProfileLayout(KernelObjectLayout):
    
    def __init__(self, escobj, **kwargs):
        func = None
        profile = kwargs.get("profile", None)
        src = kwargs.get("source", None)
        if not isinstance(escobj, esc.multilayer_obj):
            raise TypeError(f"Expected multilayer object, got {type(escobj)}")
        if profile is None:
            profile = SLD0RE_TYPE
        if profile == SLD0RE_TYPE:
            prof = esc.profile_sld0("", esc.var("z"), escobj, src)
            func = esc.real(prof)
        elif profile == SLD0IM_TYPE:
            prof = esc.profile_sld0("", esc.var("z"), escobj, src)
            func = esc.imag(prof)
        elif profile == SLDM_TYPE:
            raise NotImplementedError("SLDM profile is not implemented yet")
        units = ""
        if not src is None:
            units = esc.angstr if src.units.lower()=="angstrom" else "nm"
        k=esc.kernel("", func, multithreaded=True)
        coordinates = kwargs.get("coordinates", None)
        if 'xlabel' in kwargs:
            kwargs.pop('xlabel')
        if 'ylabel' in kwargs:
            kwargs.pop('ylabel')
        if not coordinates is None:
            kwargs.pop("coordinates")
        else:
            zmax = escobj.total_thickness*1.5
            numpoints = 100
            if zmax == 0: #if substrate only
                zmax = 200
            else:
                minthkn = escobj.min_thickness
                if minthkn==0:
                    raise ValueError("Minimum thickness is 0")
                numpoints = max(int(zmax/minthkn)*5, 50)      
                coordinates=np.linspace(-min(100, zmax), zmax, numpoints)
        super(ProfileLayout, self).__init__(k, coordinates=coordinates, 
                xlabel=f"Z[{units}]", 
                ylabel=f"{profile.upper()}[1/{units}²]", **kwargs)
        
            
            
            
            
